<?php
/**
 * 2013 - 2021 PayPlug SAS
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0).
 * It is available through the world-wide-web at this URL:
 * https://opensource.org/licenses/osl-3.0.php
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to contact@payplug.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PayPlug module to newer
 * versions in the future.
 *
 * @author    PayPlug SAS
 * @copyright 2013 - 2021 PayPlug SAS
 * @license   https://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *  International Registered Trademark & Property of PayPlug SAS
 */

require_once(dirname(__FILE__) . './../../../../config/config.inc.php');

/** Call init.php to initialize context */
require_once(_PS_MODULE_DIR_ . '../init.php');
require_once(_PS_MODULE_DIR_ . 'payplug/payplug.php');
require_once(_PS_MODULE_DIR_ . 'payplug/classes/PayplugAdmin.php');
require_once(_PS_MODULE_DIR_ . 'payplug/classes/PayplugBackward.php');

$payplug = new Payplug();

if ((int)Tools::getValue('_ajax') == 1) {
    if ((int)Tools::getValue('en') == 1 && !$payplug->getConfiguration('PAYPLUG_SHOW')) {
        $payplug->setConfiguration('PAYPLUG_SHOW', 1);
        $payplug->enable();
        die(true);
    }
    if (Tools::getIsset('en')
        && (int)Tools::getValue('en') == 0
        && $payplug->getConfiguration('PAYPLUG_SHOW')
    ) {
        $payplug->setConfiguration('PAYPLUG_SHOW', 0);
        die(true);
    }
    // todo: trnasformer en booléen si possible
    if (Tools::getIsset('db')) {
        if (Tools::getValue('db')) {
            $payplug->setConfiguration('PAYPLUG_DEBUG_MODE', 1);
        } else {
            $payplug->setConfiguration('PAYPLUG_DEBUG_MODE', 0);
        }
        die(true);
    }
    if (Tools::getValue('popin')) {
        $args = null;
        if (Tools::getValue('type') == 'confirm') {
            $sandbox = (int)Tools::getValue('sandbox');
            $embedded = (int)Tools::getValue('embedded');
            $one_click = (int)Tools::getValue('one_click');
            $oney = (int)Tools::getValue('oney');
            $installment = (int)Tools::getValue('installment');
            $deferred = (int)Tools::getValue('deferred');
            $activate = (int)Tools::getValue('activate');
            $args = array(
                'sandbox' => $sandbox,
                'embedded' => $embedded,
                'one_click' => $one_click,
                'oney' => $oney,
                'installment' => $installment,
                'deferred' => $deferred,
                'activate' => $activate,
            );
        }
        $payplug->displayPopin(Tools::getValue('type'), $args);
    }
    if (Tools::getValue('submit') == 'submitPopin_pwd') {
        $payplug->submitPopinPwd($_POST['pwd']);
    }
    if (Tools::getValue('has_live_key')) {
        die(Tools::jsonEncode(['result' => $payplug->hasLiveKey()]));
    }
    if (Tools::getValue('submit') == 'submitPopin_confirm') {
        die(Tools::jsonEncode(array('content' => 'confirm_ok')));
    }
    if (Tools::getValue('submit') == 'submitPopin_confirm_a') {
        die(Tools::jsonEncode(array('content' => 'confirm_ok_activate')));
    }
    if (Tools::getValue('submit') == 'submitPopin_desactivate') {
        die(Tools::jsonEncode(array('content' => 'confirm_ok_desactivate')));
    }
    if (Tools::getValue('submit') == 'submitPopin_abort') {
        $inst_id = Tools::getValue('inst_id');
        $id_order = Tools::getValue('id_order');
        $pay_mode = Tools::getValue('pay_mode');

        $payplug->initializeApi($pay_mode != 'LIVE');

        try {
            $abort = \Payplug\InstallmentPlan::abort($inst_id);
        } catch (Exception $e) {
            die(Tools::jsonEncode(array(
                'status' => 'error',
                'data' => $payplug->l('Cannot abort installment.')
            )));
        }

        if ($abort == 'error') {
            die(Tools::jsonEncode(array(
                'status' => 'error',
                'data' => $payplug->l('Cannot abort installment.')
            )));
        } else {
            $installment = $payplug->retrieveInstallment($inst_id);
            $new_state = $payplug->getConfiguration(
                'PAYPLUG_ORDER_STATE_ERROR' . ($installment->is_live ? '' : '_TEST')
            );

            $order = new Order((int)$id_order);
            if (Validate::isLoadedObject($order)) {
                $current_state = (int)$order->getCurrentState();
                if ($current_state != 0 && $current_state != $new_state) {
                    $history = new OrderHistory();
                    $history->id_order = (int)$order->id;
                    $history->changeIdOrderState($new_state, (int)$order->id);
                    $history->addWithemail();
                }
            }
            $reload = true;

            die(Tools::jsonEncode(array('reload' => $reload)));
        }
    }
    if (Tools::getValue('check')) {
        $content = $payplug->getCheckFieldset();
        die(Tools::jsonEncode(array('content' => $content)));
    }
    if (Tools::getValue('log')) {
        $content = $payplug->getLogin();
        die(Tools::jsonEncode(array('content' => $content)));
    }
    if (Tools::getValue('checkPremium')) {
        $api_key = $payplug->getConfiguration('PAYPLUG_LIVE_API_KEY');
        die(Tools::jsonEncode($payplug->checkPremium($api_key)));
    }
    if (Tools::getValue('refund')) {
        if (!$payplug->checkAmountToRefund(Tools::getValue('amount'))) {
            die(Tools::jsonEncode(array(
                'status' => 'error',
                'data' => $payplug->l('Incorrect amount to refund')
            )));
        } else {
            $amount = str_replace(',', '.', Tools::getValue('amount'));
            $amount = (float)($amount * 1000); // we use this trick to avoid rounding while converting to int
            $amount = (float)($amount / 10); // unless sometimes 17.90 become 17.89
            $amount = (int)$amount;
        }

        $id_order = Tools::getValue('id_order');
        $pay_id = Tools::getValue('pay_id');
        $inst_id = Tools::getValue('inst_id');
        $metadata = array(
            'Client' => (int)Tools::getValue('id_customer'),
            'reason' => 'Refunded with Prestashop'
        );
        $pay_mode = Tools::getValue('pay_mode');
        $refund = $payplug->makeRefund($pay_id, $amount, $metadata, $pay_mode, $inst_id);
        if ($refund == 'error') {
            die(Tools::jsonEncode(array(
                'status' => 'error',
                'data' => $payplug->l('Cannot refund that amount.')
            )));
        } else {
            $new_state = 7;
            $reload = false;

            if ($inst_id != null) {
                $installment = $payplug->retrieveInstallment($inst_id);
                $amount_available = 0;
                $amount_refunded_payplug = 0;
                if (isset($installment->schedule)) {
                    foreach ($installment->schedule as $schedule) {
                        if (!empty($schedule->payment_ids)) {
                            foreach ($schedule->payment_ids as $p_id) {
                                $p = \Payplug\Payment::retrieve($p_id);
                                if ($p->is_paid && !$p->is_refunded) {
                                    $amount_available += (int)($p->amount - $p->amount_refunded);
                                }
                                $amount_refunded_payplug += $p->amount_refunded;
                            }
                        }
                    }
                }
                $amount_available = (float)($amount_available / 100);
                $amount_refunded_payplug = (float)($amount_refunded_payplug / 100);
                if ((int)Tools::getValue('id_state') != 0) {
                    $new_state = (int)Tools::getValue('id_state');
                    $order = new Order((int)$id_order);
                    if (Validate::isLoadedObject($order)) {
                        $current_state = (int)$order->getCurrentState();
                        if ($current_state != 0 && $current_state != $new_state) {
                            $history = new OrderHistory();
                            $history->id_order = (int)$order->id;
                            $history->changeIdOrderState($new_state, (int)$order->id);
                            $history->addWithemail();
                        }
                    }
                    $reload = true;
                }
            } else {
                $payment = $payplug->retrievePayment($pay_id);
                if ((int)Tools::getValue('id_state') != 0) {
                    $new_state = (int)Tools::getValue('id_state');
                } elseif ((int)$payment->is_refunded == 1) {
                    $new_state = $payplug->getConfiguration(
                        'PAYPLUG_ORDER_STATE_REFUND' . ($payment->is_live ? '' : '_TEST')
                    );
                }

                if ((int)Tools::getValue('id_state') != 0 || (int)$payment->is_refunded == 1) {
                    $order = new Order((int)$id_order);
                    if (Validate::isLoadedObject($order)) {
                        $current_state = (int)$order->getCurrentState();
                        if ($current_state != 0 && $current_state != $new_state) {
                            $history = new OrderHistory();
                            $history->id_order = (int)$order->id;
                            $history->changeIdOrderState($new_state, (int)$order->id);
                            $history->addWithemail();
                        }
                    }
                    $reload = true;
                }

                $amount_refunded_payplug = ($payment->amount_refunded) / 100;
                $amount_available = ($payment->amount - $payment->amount_refunded) / 100;
            }

            $data = $payplug->getRefundData(
                $amount_refunded_payplug,
                $amount_available
            );
            die(Tools::jsonEncode(array(
                'status' => 'ok',
                'data' => $data,
                'message' => $payplug->l('Amount successfully refunded.'),
                'reload' => $reload
            )));
        }
    }
    if ((int)Tools::getValue('capture') == 1) {
        $pay_id = Tools::getValue('pay_id');
        $id_order = Tools::getValue('id_order');
        $payment = new PPPayment($pay_id);
        $capture = $payment->capture();
        $payment->refresh();
        if ($payment->resource->card->id !== null) {
            $payplug->saveCard($payment->resource);
        }
        if ($capture['code'] >= 300) {
            die(Tools::jsonEncode(array(
                'status' => 'error',
                'data' => $payplug->l('Cannot capture this payment.'),
                'message' => $capture['message'],
            )));
        } else {
            $state_addons = ($payment->resource->is_live ? '' : '_TEST');
            $new_state = $payplug->getConfiguration('PAYPLUG_ORDER_STATE_PAID' . $state_addons);

            $order = new Order((int)$id_order);
            if (Validate::isLoadedObject($order)) {
                $order->setInvoice(true);
                $current_state = (int)$order->getCurrentState();
                if ($current_state != 0 && $current_state != $new_state) {
                    $history = new OrderHistory();
                    $history->id_order = (int)$order->id;
                    $history->changeIdOrderState($new_state, (int)$order->id);
                    $history->addWithemail();
                }
            }

            die(Tools::jsonEncode(array(
                'status' => 'ok',
                'data' => '',
                'message' => $payplug->l('Payment successfully captured.'),
                'reload' => true,
            )));
        }
    }
    if ((int)Tools::getValue('cancel') == 1) {
        $pay_id = Tools::getValue('pay_id');
        $payment = new PPPayment($pay_id);
        $cancel = $payment->abort();
        $payment->refresh();
        if ($cancel['code'] >= 300) {
            die(Tools::jsonEncode(array(
                'status' => 'error',
                'data' => $payplug->l('Cannot cancel this payment.'),
                'message' => $cancel['message'],
            )));
        } else {
            die(Tools::jsonEncode(array(
                'status' => 'ok',
                'data' => '',
                'message' => $payplug->l('Payment successfully cancel.'),
                'reload' => true,
            )));
        }
    }
    if ((int)Tools::getValue('popinRefund') == 1) {
        $popin = $payplug->displayPopin('refund');
        die(Tools::jsonEncode(array('content' => $popin)));
    }
    if ((int)Tools::getValue('update') == 1) {
        $pay_id = Tools::getValue('pay_id');
        $payment = $payplug->retrievePayment($pay_id);
        $id_order = Tools::getValue('id_order');

        if ($payment->is_paid) {
            $new_state = $payplug->getConfiguration('PAYPLUG_ORDER_STATE_PAID' . ($payment->is_paid ? '_TEST' : ''));
        } elseif ((int)$payment->is_paid == 0) {
            $new_state = $payplug->getConfiguration('PAYPLUG_ORDER_STATE_ERROR' . ($payment->is_paid ? '_TEST' : ''));
        }

        $order = new Order((int)$id_order);
        if (Validate::isLoadedObject($order)) {
            $current_state = (int)$order->getCurrentState();
            if ($current_state != 0 && $current_state != $new_state) {
                $history = new OrderHistory();
                $history->id_order = (int)$order->id;
                $history->changeIdOrderState($new_state, (int)$order->id);
                $history->addWithemail();
            }
        }

        die(Tools::jsonEncode(array(
            'message' => $payplug->l('Order successfully updated.'),
            'reload' => true
        )));
    }
    if ((int)Tools::getValue('cancel')) {
        $pay_id = Tools::getValue('pay_id');
        $payment = new PPPayment($pay_id);
        $cancel = $payment->abort();
        $payment->refresh();
        if ($cancel['code'] >= 300) {
            die(Tools::jsonEncode(array(
                'status' => 'error',
                'data' => $payplug->l('Cannot cancel this payment.'),
                'message' => $cancel['message'],
            )));
        } else {
            die(Tools::jsonEncode(array(
                'status' => 'ok',
                'data' => '',
                'message' => $payplug->l('Payment successfully cancel.'),
                'reload' => true,
            )));
        }
    }
} else {
    exit;
}
